<?php
/**
 * @version			1.0.0
 * @package			Joomla
 * @subpackage		EShop
 * @author			Giang Dinh Truong
 * @copyright		Copyright (C) 2012 Ossolution Team
 * @license			GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

/**
 * Stripe Checkout payment plugin for EShop
 *
 * @author Giang Dinh Truong
 *
 */
class os_stripecheckout extends os_payment
{

	/**
	 * Constructor
	 *
	 * @param JRegistry $params
	 * @param array     $config
	 */
	public function __construct($params, $config = array())
	{
		parent::__construct($params, $config);
	}

	/**
	 * Process Payment
	 *
	 * @param array $data        	
	 */
	public function processPayment($data)
	{
	    $this->loadStripeLib();

		$siteUrl = JUri::base();
		$returnUrl = $siteUrl . 'index.php?option=com_eshop&view=checkout&layout=complete&order_number=' . $data['order_number'];
		$cancelUrl = $siteUrl . 'index.php?option=com_eshop&view=checkout&layout=cancel&id=' . $data['order_id'];
		
		$metaData['First Name']	= $data['payment_firstname'];
		$metaData['Last Name']	= $data['payment_lastname'];
		$metaData['Email']		= $data['email'];
		$metaData['Source']		= 'EShop';
		
		$description = JText::sprintf('ESHOP_PAYMENT_FOR_ORDER', $data['order_id']);

		if ($data['customer_id'] > 0)
		{
			$metaData['Customer ID']= $data['customer_id'];
		}

		$metaData['Order ID']  = $data['order_id'];

		if (EshopHelper::getConfigValue('default_currency_code') != $data['currency_code'])
		{
			$currency = new EshopCurrency();
			$amount = round($currency->convert($data['total'], EshopHelper::getConfigValue('default_currency_code'), $data['currency_code']), 2);
		}
		else
		{
			$amount = round($data['total'], 2);
		}

		try
		{
			$session = \Stripe\Checkout\Session::create([
				'mode'			=> 'payment',
				'line_items'	=> [[
					'name'		=> $description,
					'amount'	=> 100 * $amount,
					'currency'	=> $data['currency_code'],
					'quantity'	=> 1,
				]],
				'payment_intent_data'  => [
					'description' => $description,
					'metadata'    => $metaData,
				],
				'success_url'          => $returnUrl,
				'cancel_url'           => $cancelUrl,
				'client_reference_id'  => $this->params->get('order_prefix', 'ESHOP') . $data['order_id'],
				'customer_email'       => $data['email']
			]);
			
			//Get redirect heading
			$language    = JFactory::getLanguage();
			$languageKey = 'ESHOP_WAIT_' . strtoupper(substr($this->name, 3));

			if ($language->hasKey($languageKey))
			{
				$redirectHeading = JText::_($languageKey);
			}
			else
			{
				$redirectHeading = JText::sprintf('ESHOP_REDIRECT_HEADING', $this->getTitle());
			}
			?>
			<div class="eshop-heading"><?php echo $redirectHeading ?></div>
            <script src="https://js.stripe.com/v3/"></script>
            <script type="text/javascript">
                var stripe = Stripe('<?php echo $this->params->get('stripe_public_key'); ?>');

                stripe.redirectToCheckout({
                    // Make the id field from the Checkout Session creation API response
                    // available to this file, so you can provide it as parameter here
                    // instead of the {{CHECKOUT_SESSION_ID}} placeholder.
                    sessionId: '<?php echo $session->id; ?>'
                }).then(function (result) {
                    // If `redirectToCheckout` fails due to a browser or network
                    // error, display the localized error message to your customer
                    // using `result.error.message`.
                    alert(result.error.message);
                });
            </script>
			<?php
		}
		catch (Exception $e)
		{
			JFactory::getSession()->set('omnipay_payment_error_reason', $e->getMessage());

			JFactory::getApplication()->redirect(JRoute::_('index.php?option=com_eshop&view=checkout&layout=failure', false));
		}
	}

	public function verifyPayment()
	{
		if (!$this->validate())
		{
			return;
		}

		$id            = $this->postData['id'];

		$row = JTable::getInstance('Eshop', 'Order');

		if (!$row->load($id))
		{
			$this->logGatewayData('Invalid Record ID:' . $id);
			
			return false;
		}
		
		// Validate payment status
		if ($row->order_status_id == EshopHelper::getConfigValue('complete_status_id'))
		{
			$this->logGatewayData('Invalid Status, already published before');
			
			return false;
		}
		
		$row->transaction_id = $this->postData['transaction_id'];
		$row->order_status_id = EshopHelper::getConfigValue('complete_status_id');
		$row->store();
		EshopHelper::completeOrder($row);
		JPluginHelper::importPlugin('eshop');
		JFactory::getApplication()->triggerEvent('onAfterCompleteOrder', array($row));
		
		//Send confirmation email here
		if (EshopHelper::getConfigValue('order_alert_mail'))
		{
			EshopHelper::sendEmails($row);
		}
		
		return true;
	}

	/**
	 *
	 */
	protected function validate()
	{
	    $this->loadStripeLib();

		$endpoint_secret = $this->params->get('endpoint_secret');

		$payload = @file_get_contents('php://input');
		$sig_header = $_SERVER['HTTP_STRIPE_SIGNATURE'];
		$event = null;

		try {
			$event = \Stripe\Webhook::constructEvent(
				$payload, $sig_header, $endpoint_secret
			);
		} catch(\UnexpectedValueException $e) {
			// Invalid payload
			http_response_code(400); // PHP 5.4 or greater

            return false;
		} catch(\Stripe\Error\SignatureVerification $e) {
			// Invalid signature
			http_response_code(400); // PHP 5.4 or greater

            return false;
		}

		if ($event->type == 'checkout.session.completed') {

			$session = $event->data->object;
			$paymentIntent = \Stripe\PaymentIntent::retrieve($session->payment_intent);
			
			$orderPrefix = $this->params->get('order_prefix', 'ESHOP');

			if ($orderPrefix && strpos($session->client_reference_id, $orderPrefix) === false)
			{
				return false;
			}

			$this->postData['id'] = (int) substr($session->client_reference_id, strlen($orderPrefix));
			
			$transactionId =  '';

			foreach ($paymentIntent->charges as $charge)
            {
                $transactionId = $charge->id;
            }

            $this->postData['transaction_id'] = $transactionId;
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Load Stripe library, set API Key and make the API ready to be used
	 */
	protected function loadStripeLib()
	{
		if (!class_exists('\Stripe\Stripe'))
		{
			require_once JPATH_ROOT . '/components/com_eshop/plugins/payment/stripecheckout/init.php';
		}

		\Stripe\Stripe::setApiKey($this->params->get('stripe_api_key'));
	}
}